@@ -114,7 +114,7 @@ send_symref(struct got_reference *symref, struct got_object_id *target_id,
 	memset(&isymref, 0, sizeof(isymref));
 	isymref.name_len = strlen(refname);
 	isymref.target_len = strlen(target);
-	memcpy(isymref.target_id, target_id->sha1, sizeof(isymref.target_id));
+	memcpy(isymref.target_id, target_id->hash, sizeof(isymref.target_id));
 
 	len = sizeof(isymref) + isymref.name_len + isymref.target_len;
 	if (len > MAX_IMSGSIZE - IMSG_HEADER_SIZE) {
@@ -184,7 +184,7 @@ send_peeled_tag_ref(struct got_reference *ref, struct got_object *obj,
 	}
 
 	/* Keep in sync with struct gotd_imsg_ref definition. */
-	if (imsg_add(wbuf, id->sha1, SHA1_DIGEST_LENGTH) == -1) {
+	if (imsg_add(wbuf, id->hash, SHA1_DIGEST_LENGTH) == -1) {
 		err = got_error_from_errno("imsg_add REF");
 		goto done;
 	}
@@ -232,7 +232,7 @@ send_ref(struct got_reference *ref, struct imsgbuf *ibuf)
 	}
 
 	/* Keep in sync with struct gotd_imsg_ref definition. */
-	if (imsg_add(wbuf, id->sha1, SHA1_DIGEST_LENGTH) == -1)
+	if (imsg_add(wbuf, id->hash, SHA1_DIGEST_LENGTH) == -1)
 		return got_error_from_errno("imsg_add REF");
 	if (imsg_add(wbuf, &namelen, sizeof(namelen)) == -1)
 		return got_error_from_errno("imsg_add REF");
@@ -263,15 +263,10 @@ list_refs(struct imsg *imsg)
 	size_t datalen;
 	struct gotd_imsg_reflist irefs;
 	struct imsgbuf ibuf;
-	int client_fd;
 	struct got_object_id *head_target_id = NULL;
 
 	TAILQ_INIT(&refs);
 
-	client_fd = imsg_get_fd(imsg);
-	if (client_fd == -1)
-		return got_error(GOT_ERR_PRIVSEP_NO_FD);
-
 	datalen = imsg->hdr.len - IMSG_HEADER_SIZE;
 	if (datalen != 0)
 		return got_error(GOT_ERR_PRIVSEP_LEN);
@@ -282,9 +277,13 @@ list_refs(struct imsg *imsg)
 	}
 	repo_read.refs_listed = 1;
 
-	client->fd = client_fd;
+	client->fd = imsg_get_fd(imsg);
+	if (client->fd == -1)
+		return got_error(GOT_ERR_PRIVSEP_NO_FD);
 
-	imsg_init(&ibuf, client_fd);
+	if (imsgbuf_init(&ibuf, client->fd) == -1)
+		return got_error_from_errno("imsgbuf_init");
+	imsgbuf_allow_fdpass(&ibuf);
 
 	err = got_ref_list(&refs, repo_read.repo, "",
 	    got_ref_cmp_by_name, NULL);
@@ -364,7 +363,7 @@ list_refs(struct imsg *imsg)
 	err = gotd_imsg_flush(&ibuf);
 done:
 	got_ref_list_free(&refs);
-	imsg_clear(&ibuf);
+	imsgbuf_clear(&ibuf);
 	return err;
 }
 
@@ -414,13 +413,15 @@ recv_want(struct imsg *imsg)
 	memcpy(&iwant, imsg->data, sizeof(iwant));
 
 	memset(&id, 0, sizeof(id));
-	memcpy(id.sha1, iwant.object_id, SHA1_DIGEST_LENGTH);
+	memcpy(id.hash, iwant.object_id, SHA1_DIGEST_LENGTH);
 
 	if (log_getverbose() > 0 &&
 	    got_object_id_hex(&id, hex, sizeof(hex)))
 		log_debug("client wants %s", hex);
 
-	imsg_init(&ibuf, client->fd);
+	if (imsgbuf_init(&ibuf, client->fd) == -1)
+		return got_error_from_errno("imsgbuf_init");
+	imsgbuf_allow_fdpass(&ibuf);
 
 	err = got_object_get_type(&obj_type, repo_read.repo, &id);
 	if (err)
@@ -437,7 +438,7 @@ recv_want(struct imsg *imsg)
 	}
 
 	gotd_imsg_send_ack(&id, &ibuf, PROC_REPO_READ, repo_read.pid);
-	imsg_clear(&ibuf);
+	imsgbuf_clear(&ibuf);
 	return err;
 }
 
@@ -459,13 +460,15 @@ recv_have(struct imsg *imsg)
 	memcpy(&ihave, imsg->data, sizeof(ihave));
 
 	memset(&id, 0, sizeof(id));
-	memcpy(id.sha1, ihave.object_id, SHA1_DIGEST_LENGTH);
+	memcpy(id.hash, ihave.object_id, SHA1_DIGEST_LENGTH);
 
 	if (log_getverbose() > 0 &&
 	    got_object_id_hex(&id, hex, sizeof(hex)))
 		log_debug("client has %s", hex);
 
-	imsg_init(&ibuf, client->fd);
+	if (imsgbuf_init(&ibuf, client->fd) == -1)
+		return got_error_from_errno("imsgbuf_init");
+	imsgbuf_allow_fdpass(&ibuf);
 
 	err = got_object_get_type(&obj_type, repo_read.repo, &id);
 	if (err) {
@@ -492,7 +495,7 @@ recv_have(struct imsg *imsg)
 
 	gotd_imsg_send_ack(&id, &ibuf, PROC_REPO_READ, repo_read.pid);
 done:
-	imsg_clear(&ibuf);
+	imsgbuf_clear(&ibuf);
 	return err;
 }
 
@@ -600,8 +603,8 @@ send_packfile(struct imsg *imsg, struct gotd_imsgev *iev)
 {
 	const struct got_error *err = NULL;
 	struct repo_read_client *client = &repo_read_client;
-	uint8_t packsha1[SHA1_DIGEST_LENGTH];
-	char hex[SHA1_DIGEST_STRING_LENGTH];
+	struct got_object_id packhash;
+	char hex[GOT_HASH_DIGEST_STRING_MAXLEN];
 	FILE *delta_cache = NULL;
 	struct imsgbuf ibuf;
 	struct repo_read_pack_progress_arg pa;
@@ -619,7 +622,9 @@ send_packfile(struct imsg *imsg, struct gotd_imsgev *iev)
 	if (client->delta_cache_fd == -1 || client->pack_pipe == -1)
 		return got_error(GOT_ERR_PRIVSEP_NO_FD);
 
-	imsg_init(&ibuf, client->fd);
+	if (imsgbuf_init(&ibuf, client->fd) == -1)
+		return got_error_from_errno("imsgbuf_init");
+	imsgbuf_allow_fdpass(&ibuf);
 
 	delta_cache = fdopen(client->delta_cache_fd, "w+");
 	if (delta_cache == NULL) {
@@ -641,7 +646,7 @@ send_packfile(struct imsg *imsg, struct gotd_imsgev *iev)
 	if (err)
 		goto done;
 
-	err = got_pack_create(packsha1, client->pack_pipe, delta_cache,
+	err = got_pack_create(&packhash, client->pack_pipe, delta_cache,
 	    have_ids.ids, have_ids.nids, want_ids.ids, want_ids.nids,
 	    repo_read.repo, 0, 1, 0, pack_progress, &pa, &rl,
 	    check_cancelled, NULL);
@@ -649,7 +654,8 @@ send_packfile(struct imsg *imsg, struct gotd_imsgev *iev)
 		goto done;
 
 	if (log_getverbose() > 0 &&
-	    got_sha1_digest_to_str(packsha1, hex, sizeof(hex)))
+	    got_hash_digest_to_str(packhash.hash, hex, sizeof(hex),
+	    packhash.algo))
 		log_debug("sent pack-%s.pack", hex);
 
 	if (gotd_imsg_compose_event(iev, GOTD_IMSG_PACKFILE_DONE,
@@ -662,7 +668,7 @@ done:
 	client->delta_cache_fd = -1;
 	if (delta_cache != NULL && fclose(delta_cache) == EOF && err == NULL)
 		err = got_error_from_errno("fclose");
-	imsg_clear(&ibuf);
+	imsgbuf_clear(&ibuf);
 	free(want_ids.ids);
 	free(have_ids.ids);
 	return err;
@@ -680,18 +686,16 @@ repo_read_dispatch_session(int fd, short event, void *arg)
 	struct repo_read_client *client = &repo_read_client;
 
 	if (event & EV_READ) {
-		if ((n = imsg_read(ibuf)) == -1 && errno != EAGAIN)
+		if ((n = imsgbuf_read(ibuf)) == -1)
 			fatal("imsg_read error");
 		if (n == 0)	/* Connection closed. */
 			shut = 1;
 	}
 
 	if (event & EV_WRITE) {
-		n = msgbuf_write(&ibuf->w);
-		if (n == -1 && errno != EAGAIN)
-			fatal("msgbuf_write");
-		if (n == 0)	/* Connection closed. */
-			shut = 1;
+		err = gotd_imsg_flush(ibuf);
+		if (err)
+			fatalx("%s", err->msg);
 	}
 
 	while (err == NULL && check_cancelled(NULL) == NULL) {
@@ -778,7 +782,9 @@ recv_connect(struct imsg *imsg)
 	if (repo_read.session_fd == -1)
 		return got_error(GOT_ERR_PRIVSEP_NO_FD);
 
-	imsg_init(&iev->ibuf, repo_read.session_fd);
+	if (imsgbuf_init(&iev->ibuf, repo_read.session_fd) == -1)
+		return got_error_from_errno("imsgbuf_init");
+	imsgbuf_allow_fdpass(&iev->ibuf);
 	iev->handler = repo_read_dispatch_session;
 	iev->events = EV_READ;
 	iev->handler_arg = NULL;
@@ -801,18 +807,16 @@ repo_read_dispatch(int fd, short event, void *arg)
 	struct repo_read_client *client = &repo_read_client;
 
 	if (event & EV_READ) {
-		if ((n = imsg_read(ibuf)) == -1 && errno != EAGAIN)
+		if ((n = imsgbuf_read(ibuf)) == -1)
 			fatal("imsg_read error");
 		if (n == 0)	/* Connection closed. */
 			shut = 1;
 	}
 
 	if (event & EV_WRITE) {
-		n = msgbuf_write(&ibuf->w);
-		if (n == -1 && errno != EAGAIN)
-			fatal("msgbuf_write");
-		if (n == 0)	/* Connection closed. */
-			shut = 1;
+		err = gotd_imsg_flush(ibuf);
+		if (err)
+			fatalx("%s", err->msg);
 	}
 
 	while (err == NULL && check_cancelled(NULL) == NULL) {
@@ -885,6 +889,11 @@ repo_read_main(const char *title, const char *repo_path,
 		    "bare git repository required");
 		goto done;
 	}
+	if (got_repo_get_object_format(repo_read.repo) != GOT_HASH_SHA1) {
+		err = got_error_msg(GOT_ERR_NOT_IMPL,
+		    "sha256 object IDs unsupported in network protocol");
+		goto done;
+	}
 
 	got_repo_temp_fds_set(repo_read.repo, temp_fds);
 
@@ -893,7 +902,11 @@ repo_read_main(const char *title, const char *repo_path,
 	signal(SIGPIPE, SIG_IGN);
 	signal(SIGHUP, SIG_IGN);
 
-	imsg_init(&iev.ibuf, GOTD_FILENO_MSG_PIPE);
+	if (imsgbuf_init(&iev.ibuf, GOTD_FILENO_MSG_PIPE) == -1) {
+		err = got_error_from_errno("imsgbuf_init");
+		goto done;
+	}
+	imsgbuf_allow_fdpass(&iev.ibuf);
 	iev.handler = repo_read_dispatch;
 	iev.events = EV_READ;
 	iev.handler_arg = NULL;
