@@ -33,13 +33,13 @@
 #include "got_path.h"
 
 #include "got_lib_delta.h"
+#include "got_lib_hash.h"
 #include "got_lib_object.h"
 #include "got_lib_object_cache.h"
 #include "got_lib_object_parse.h"
 #include "got_lib_pack.h"
 #include "got_lib_repository.h"
 #include "got_lib_inflate.h"
-#include "got_lib_hash.h"
 
 const struct got_error *
 got_object_open_packed(struct got_object **obj, struct got_object_id *id,
@@ -143,11 +143,8 @@ got_object_open(struct got_object **obj, struct got_repository *repo,
 		return NULL;
 
 	err = got_object_open_loose_fd(&fd, id, repo);
-	if (err) {
-		if (err->code == GOT_ERR_ERRNO && errno == ENOENT)
-			err = got_error_no_obj(id);
+	if (err)
 		return err;
-	}
 
 	err = got_object_read_header(obj, fd);
 	if (err)
@@ -427,7 +424,8 @@ open_commit(struct got_commit_object **commit,
 		got_object_close(obj);
 		if (err)
 			goto done;
-		err = got_object_parse_commit(commit, buf, len);
+		err = got_object_parse_commit(commit, buf, len,
+		    got_repo_get_object_format(repo));
 		free(buf);
 	} else if (err->code == GOT_ERR_NO_OBJ) {
 		int fd;
@@ -524,7 +522,8 @@ open_tree(struct got_tree_object **tree,
 		if (err)
 			goto done;
 		err = got_object_parse_tree(&entries, &nentries,
-		    &nentries_alloc, buf, len);
+		    &nentries_alloc, buf, len,
+		    got_repo_get_object_format(repo));
 		if (err)
 			goto done;
 	} else if (err->code == GOT_ERR_NO_OBJ) {
@@ -558,13 +557,14 @@ open_tree(struct got_tree_object **tree,
 	for (i = 0; i < nentries; i++) {
 		struct got_parsed_tree_entry *pe = &entries[i];
 		struct got_tree_entry *te = &(*tree)->entries[i];
-	
+
 		if (strlcpy(te->name, pe->name,
 		    sizeof(te->name)) >= sizeof(te->name)) {
 			err = got_error(GOT_ERR_NO_SPACE);
 			goto done;
 		}
-		memcpy(te->id.sha1, pe->id, SHA1_DIGEST_LENGTH);
+		memcpy(te->id.hash, pe->id, pe->digest_len);
+		te->id.algo = pe->algo;
 		te->mode = pe->mode;
 		te->idx = i;
 	}
@@ -637,7 +637,7 @@ read_packed_blob(uint8_t **outbuf, size_t *size, size_t *hdrlen,
 
 	if (blob_size <= GOT_DELTA_RESULT_SIZE_CACHED_MAX) {
 		err = got_packfile_extract_object_to_mem(outbuf, size,
-		    obj, pack); 
+		    obj, pack);
 	} else {
 		/*
 		 * XXX This uses 3 file extra descriptors for no good reason.
@@ -682,7 +682,7 @@ read_blob(uint8_t **outbuf, size_t *size, size_t *hdrlen, int outfd, int infd,
 	struct got_inflate_checksum csum;
 	struct got_hash ctx;
 
-	got_hash_init(&ctx, GOT_HASH_SHA1);
+	got_hash_init(&ctx, got_repo_get_object_format(repo));
 	memset(&csum, 0, sizeof(csum));
 	csum.output_ctx = &ctx;
 
@@ -801,7 +801,7 @@ open_blob(struct got_blob_object **blob, struct got_repository *repo,
 		goto done;
 	}
 
-	if (outbuf) {
+	if (outbuf && size > 0) {
 		(*blob)->f = fmemopen(outbuf, size, "rb");
 		if ((*blob)->f == NULL) {
 			err = got_error_from_errno("fmemopen");
@@ -920,7 +920,8 @@ open_tag(struct got_tag_object **tag, struct got_repository *repo,
 		got_object_close(obj);
 		if (err)
 			goto done;
-		err = got_object_parse_tag(tag, buf, len);
+		err = got_object_parse_tag(tag, buf, len,
+		    got_repo_get_object_format(repo));
 		free(buf);
 	} else if (err->code == GOT_ERR_NO_OBJ) {
 		int fd;
