@@ -560,13 +560,13 @@ get_fileindex_path(char **fileindex_path, struct got_worktree *worktree)
 
 static const struct got_error *
 open_fileindex(struct got_fileindex **fileindex, char **fileindex_path,
-    struct got_worktree *worktree)
+    struct got_worktree *worktree, enum got_hash_algorithm algo)
 {
 	const struct got_error *err = NULL;
 	FILE *index = NULL;
 
 	*fileindex_path = NULL;
-	*fileindex = got_fileindex_alloc();
+	*fileindex = got_fileindex_alloc(algo);
 	if (*fileindex == NULL)
 		return got_error_from_errno("got_fileindex_alloc");
 
@@ -579,7 +579,7 @@ open_fileindex(struct got_fileindex **fileindex, char **fileindex_path,
 		if (errno != ENOENT)
 			err = got_error_from_errno2("fopen", *fileindex_path);
 	} else {
-		err = got_fileindex_read(*fileindex, index);
+		err = got_fileindex_read(*fileindex, index, algo);
 		if (fclose(index) == EOF && err == NULL)
 			err = got_error_from_errno("fclose");
 	}
@@ -1101,7 +1101,7 @@ find_missing_children(void *arg, struct got_fileindex_entry *ie)
 	}
 
 	if (got_path_is_child(ie->path, a->parent_path, a->parent_len))
-		err = got_pathlist_append(a->children, ie->path, NULL);
+		err = got_pathlist_insert(NULL, a->children, ie->path, NULL);
 
 	return err;
 }
@@ -2142,22 +2142,20 @@ update_fileindex_after_commit(struct got_worktree *worktree,
 
 				err = got_fileindex_entry_update(ie,
 				    worktree->root_fd, relpath,
-				    ct->staged_blob_id->sha1,
-				    new_base_commit_id->sha1,
+				    ct->staged_blob_id, new_base_commit_id,
 				    !have_staged_files);
 			} else
 				err = got_fileindex_entry_update(ie,
 				    worktree->root_fd, relpath,
-				    ct->blob_id->sha1,
-				    new_base_commit_id->sha1,
+				    ct->blob_id, new_base_commit_id,
 				    !have_staged_files);
 		} else {
 			err = got_fileindex_entry_alloc(&ie, pe->path);
 			if (err)
 				goto done;
 			err = got_fileindex_entry_update(ie,
-			    worktree->root_fd, relpath, ct->blob_id->sha1,
-			    new_base_commit_id->sha1, 1);
+			    worktree->root_fd, relpath, ct->blob_id,
+			    new_base_commit_id, 1);
 			if (err) {
 				got_fileindex_entry_free(ie);
 				goto done;
@@ -2966,7 +2964,8 @@ got_worktree_cvg_commit(struct got_object_id **new_commit_id,
 	if (err)
 		goto done;
 
-	err = open_fileindex(&fileindex, &fileindex_path, worktree);
+	err = open_fileindex(&fileindex, &fileindex_path, worktree,
+	    got_repo_get_object_format(repo));
 	if (err)
 		goto done;
 
@@ -3065,7 +3064,7 @@ got_worktree_cvg_commit(struct got_object_id **new_commit_id,
 		goto done;
 	}
 
-	err = got_pathlist_append(&commit_reflist, commit_refname,
+	err = got_pathlist_insert(&pe, &commit_reflist, commit_refname,
 	    head_refname);
 	if (err)
 		goto done;
