@@ -864,6 +864,7 @@ main(int argc, char **argv)
 	struct got_imsg_fetch_wanted_branch wbranch;
 	struct got_imsg_fetch_wanted_ref wref;
 	size_t datalen, i;
+	struct got_pathlist_entry *new;
 	char *remote_head = NULL, *worktree_branch = NULL;
 #if 0
 	static int attached;
@@ -875,7 +876,12 @@ main(int argc, char **argv)
 	TAILQ_INIT(&wanted_branches);
 	TAILQ_INIT(&wanted_refs);
 
-	imsg_init(&ibuf, GOT_IMSG_FD_CHILD);
+	if (imsgbuf_init(&ibuf, GOT_IMSG_FD_CHILD) == -1) {
+		warn("imsgbuf_init");
+		return 1;
+	}
+	imsgbuf_allow_fdpass(&ibuf);
+
 #ifndef PROFILE
 	/* revoke access to most system calls */
 	if (pledge("stdio recvfd", NULL) == -1) {
@@ -984,11 +990,12 @@ main(int argc, char **argv)
 			goto done;
 		}
 		memcpy(id, &href.id, sizeof(*id));
-		err = got_pathlist_append(&have_refs, refname, id);
-		if (err) {
+		err = got_pathlist_insert(&new, &have_refs, refname, id);
+		if (err || new == NULL) {
 			free(refname);
 			free(id);
-			goto done;
+			if (err)
+				goto done;
 		}
 
 		imsg_free(&imsg);
@@ -1026,10 +1033,11 @@ main(int argc, char **argv)
 			goto done;
 		}
 
-		err = got_pathlist_append(&wanted_branches, refname, NULL);
-		if (err) {
+		err = got_pathlist_insert(&new, &wanted_branches, refname, NULL);
+		if (err || new == NULL) {
 			free(refname);
-			goto done;
+			if (err)
+				goto done;
 		}
 
 		imsg_free(&imsg);
@@ -1066,7 +1074,7 @@ main(int argc, char **argv)
 			goto done;
 		}
 
-		err = got_pathlist_append(&wanted_refs, refname, NULL);
+		err = got_pathlist_insert(NULL, &wanted_refs, refname, NULL);
 		if (err) {
 			free(refname);
 			goto done;
@@ -1113,6 +1121,7 @@ done:
 	if (err != NULL) {
 		fprintf(stderr, "%s: %s\n", getprogname(), err->msg);
 		got_privsep_send_error(&ibuf, err);
+		exit(1);
 	}
 
 	exit(0);
