@@ -20,6 +20,7 @@
 #include <sys/mman.h>
 #include <sys/uio.h>
 
+#include <err.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -81,11 +82,11 @@ main(int argc, char **argv)
 	const struct got_error *err = NULL, *close_err;
 	struct imsgbuf ibuf;
 	struct imsg imsg;
+	struct got_object_id pack_hash;
 	size_t i;
 	int idxfd = -1, tmpfd = -1;
 	FILE *tmpfiles[3];
 	struct got_pack pack;
-	uint8_t pack_hash[SHA1_DIGEST_LENGTH];
 	off_t packfile_size;
 	struct got_ratelimit rl;
 #if 0
@@ -105,7 +106,11 @@ main(int argc, char **argv)
 	if (err)
 		goto done;
 
-	imsg_init(&ibuf, GOT_IMSG_FD_CHILD);
+	if (imsgbuf_init(&ibuf, GOT_IMSG_FD_CHILD) == -1) {
+		warn("imsgbuf_init");
+		return 1;
+	}
+	imsgbuf_allow_fdpass(&ibuf);
 #ifndef PROFILE
 	/* revoke access to most system calls */
 	if (pledge("stdio recvfd", NULL) == -1) {
@@ -139,8 +144,9 @@ main(int argc, char **argv)
 		err = got_error(GOT_ERR_PRIVSEP_LEN);
 		goto done;
 	}
-	memcpy(pack_hash, imsg.data, sizeof(pack_hash));
+	memcpy(&pack_hash, imsg.data, sizeof(pack_hash));
 	pack.fd = imsg_get_fd(&imsg);
+	pack.algo = pack_hash.algo;
 
 	err = got_privsep_recv_imsg(&imsg, &ibuf, 0);
 	if (err)
@@ -205,7 +211,7 @@ main(int argc, char **argv)
 	}
 #endif
 	err = got_pack_index(&pack, idxfd, tmpfiles[0], tmpfiles[1],
-	    tmpfiles[2], pack_hash, send_index_pack_progress, &ibuf, &rl);
+	    tmpfiles[2], &pack_hash, send_index_pack_progress, &ibuf, &rl);
 done:
 	close_err = got_pack_close(&pack);
 	if (close_err && err == NULL)
